@@ -102,6 +102,7 @@ sub ready{
     }
     $self->emit("after_load_plugin");
     #接收消息
+    $self->on(poll_over=>sub{$_[0]->_recv_message();});
     $self->info("开始接收消息...\n");
     $self->_recv_message();
     $Mojo::Webqq::Client::CLIENT_COUNT++;
@@ -128,6 +129,8 @@ sub relogin{
     $self->ua->cookie_jar->empty;
     $self->model_update_failure_count(0);
     $self->poll_failure_count(0);
+    $self->csrf_token(undef);
+    $self->model_ext(0);
 
     $self->user(+{});
     $self->friend([]);
@@ -215,7 +218,9 @@ sub login {
             $self->info("帐号(" . $self->qq . ")登录成功\n");
             $self->update_user;
             $self->update_friend;
+            $self->update_friend_ext;
             $self->update_group;
+            $self->update_group_ext;
             $self->update_discuss;
             $self->update_recent;
 
@@ -275,8 +280,8 @@ sub mail{
         push @data,("From: $opt{from}","To: $opt{to}");
         push @data,"Cc: $opt{cc}" if defined $opt{cc};
         require MIME::Base64;
-        push @data,"Subject: =?UTF-8?B?" . MIME::Base64::encode_base64($opt{subject},"") . "?=";
         my $charset = defined $opt{charset}?$opt{charset}:"UTF-8";
+        push @data,"Subject: =?$charset?B?" . MIME::Base64::encode_base64($opt{subject},"") . "?=";
         if(defined $opt{text}){
             push @data,("Content-Type: text/plain; charset=$charset",'',$opt{text});
         }
