@@ -442,7 +442,7 @@ sub parse_receive_msg {
                             }
                         });
                     };
-                    if($info{type} eq 'share-file'){
+                    if(defined $info{type} and $info{type} eq 'share-file'){
                         my $msg = {
                             type        => "group_message",
                             msg_id      => $m->{value}{msg_id},
@@ -586,7 +586,7 @@ sub msg_put{
                 $sender = $self->new_friend(
                     id          =>  $msg->{sender_id},
                     nick        =>  "昵称未知",
-                    categorie   =>  "陌生人",
+                    category    =>  "陌生人",
                 );
                 $self->add_friend($sender,1);
             }
