@@ -11,15 +11,80 @@ use Mojo::Webqq::Model::Remote::_get_user_info;
 use Mojo::Webqq::Model::Remote::get_single_long_nick;
 use Mojo::Webqq::Model::Remote::get_qq_from_id;
 use Mojo::Webqq::Model::Remote::_get_user_friends;
+use Mojo::Webqq::Model::Remote::_get_user_friends_ext;
 use Mojo::Webqq::Model::Remote::_get_friends_state;
 use Mojo::Webqq::Model::Remote::_get_group_list_info;
+use Mojo::Webqq::Model::Remote::_get_group_list_info_ext;
 use Mojo::Webqq::Model::Remote::_get_group_info;
+use Mojo::Webqq::Model::Remote::_get_group_info_ext;
 use Mojo::Webqq::Model::Remote::_get_discuss_info;
 use Mojo::Webqq::Model::Remote::_get_discuss_list_info;
 use Mojo::Webqq::Model::Remote::_get_recent_info;
+use Mojo::Webqq::Model::Remote::_invite_friend;
+use Mojo::Webqq::Model::Remote::_set_group_admin;
+use Mojo::Webqq::Model::Remote::_remove_group_admin;
+use Mojo::Webqq::Model::Remote::_kick_group_member;
+use Mojo::Webqq::Model::Remote::_set_group_member_card;
 
 use base qw(Mojo::Webqq::Base);
 
+sub hash {
+    my $self = shift;
+    my $ptwebqq = shift;
+    my $uin = shift;
+
+    $uin .= "";
+    my @N;
+    for(my $T =0;$T<length($ptwebqq);$T++){
+        $N[$T % 4] ^= ord(substr($ptwebqq,$T,1));
+    }
+    my @U = ("EC", "OK");
+    my @V;
+    $V[0] =  $uin >> 24 & 255 ^ ord(substr($U[0],0,1));
+    $V[1] =  $uin >> 16 & 255 ^ ord(substr($U[0],1,1));
+    $V[2] =  $uin >> 8  & 255 ^ ord(substr($U[1],0,1));
+    $V[3] =  $uin       & 255 ^ ord(substr($U[1],1,1));
+    @U = ();
+    for(my $T=0;$T<8;$T++){
+        $U[$T] = $T%2==0?$N[$T>>1]:$V[$T>>1]; 
+    }
+    @N = ("0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F");
+    my $V = "";
+    for(my $T=0;$T<@U;$T++){
+        $V .= $N[$U[$T] >> 4 & 15];
+        $V .= $N[$U[$T] & 15];
+    }
+
+    return $V;
+}
+
+sub is_support_model_ext {
+    my $self = shift;
+    return 1 if $self->model_ext;
+    my $ret = $self->search_cookie("p_skey") && $self->search_cookie("skey");
+    $self->model_ext($ret || 0);
+    return $ret;
+}
+sub get_csrf_token {
+    my $self = shift;
+    if(not $self->is_support_model_ext){
+        $self->error("当前不支持获取扩展信息，无法获取CSRF Token");
+        return;
+    }
+    return $self->csrf_token if defined $self->csrf_token;
+    my $t = $self->search_cookie("skey");
+    my $n = 0;
+    my $o=length($t);
+    my $r;
+    if($t){
+        for($r=5381;$o>$n;$n++){
+            $r += ($r<<5) + ord(substr($t,$n,1));
+        }
+        my $token = 2147483647 & $r;
+        $self->csrf_token($token);
+        return $token;
+    } 
+}
 sub each_friend{
     my $self = shift;
     my $callback = shift;
@@ -99,6 +164,24 @@ sub add_friend {
     return $self;
 }
 
+sub update_friend_ext {
+    my $self = shift;
+    return if not $self->is_support_model_ext;
+    $self->info("更新好友扩展信息...\n");
+    my $friends_ext_info = $self->_get_user_friends_ext();
+    if(defined $friends_ext_info and ref $friends_ext_info eq "ARRAY"){
+        my(undef,$ext)=$self->array_unique($friends_ext_info,sub{"$_[0]->{nick}|$_[0]->{category}"});
+        my $unique_friend = $self->array_unique($self->friend,sub{$_[0]->nick . "|" . $_[0]->category});
+        for(@$unique_friend){
+            my $id = $_->nick . "|" . $_->category;
+            next if not exists $ext->{$id};
+            $_->{qq} = $ext->{$id}{qq};
+        }
+    }
+    else{
+        $self->warn("更新好友扩展信息失败");
+    }
+}
 sub update_friend {
     my $self = shift;
     my $friend = shift;
@@ -128,7 +211,6 @@ sub update_friend {
     }
     else{$self->warn("更新好友信息失败\n");$self->emit("model_update_fail");}
 }
-
 sub search_friend {
     my $self = shift;
     my %p = @_;
@@ -164,6 +246,49 @@ sub add_group{
     return $self;
 }
 
+sub update_group_ext {
+    my $self = shift;
+    return if not $self->is_support_model_ext;
+    $self->info("更新群扩展信息...\n");
+    my $group_list_ext = $self->_get_group_list_info_ext();
+    if(defined $group_list_ext and ref $group_list_ext eq "ARRAY"){
+        for my $g(@$group_list_ext){
+            my $group_info_ext = $self->_get_group_info_ext($g->{gnumber});
+            if(defined $group_info_ext){
+                for(keys %$group_info_ext){
+                    $g->{$_} = $group_info_ext->{$_}; 
+                }  
+            }
+            else{$self->warn("更新[ $g->{gname} ]扩展信息失败")}
+        }
+        my(undef,$gext)= $self->array_unique($group_list_ext,sub{$_[0]->{gname} . $_[0]->{member_count}});
+        my $unique_group = $self->array_unique($self->group,sub{$_[0]->gname . @{$_[0]->member}});
+        for my $g(@$unique_group){ 
+            my $id = $g->gname . @{$g->member};
+            next if not exists $gext->{$id};
+            $g->{gtype} = $gext->{$id}{gtype};
+            $g->{gnumber} = $gext->{$id}{gnumber};
+
+            my(undef,$mext) = $self->array_unique($gext->{$id}{member},sub{ $_[0]->{nick} . (defined($_[0]->{card})?$_[0]->{card}:"") . $_[0]->{gender}});
+            my $unique_member = $self->array_unique($g->member,sub{$_[0]->nick . (defined($_[0]->card)?$_[0]->card:"") . $_[0]->gender});
+            for(@$unique_member){
+                my $id = $_->nick . (defined($_->card)?$_->card:"") . $_->gender;
+                next if not exists $mext->{$id}; 
+                $_->{qage} = $mext->{$id}{qage};
+                $_->{level} = $mext->{$id}{level};
+                $_->{bad_record} = $mext->{$id}{bad_record};
+                $_->{qq} = $mext->{$id}{qq};
+                $_->{role} = $mext->{$id}{role};
+                $_->{join_time} = $mext->{$id}{join_time};
+                $_->{last_speak_time} = $mext->{$id}{last_speak_time};
+                $_->{gtype} = $g->gtype;
+                $_->{gnumber} = $g->gnumber;
+            }
+        }
+    }
+    else{$self->warn("更新群扩展信息失败\n");}
+
+}
 sub update_group {
     my $self = shift;
     my $group = shift;
@@ -172,7 +297,7 @@ sub update_group {
         $self->info("更新群 [ " . $group->gname .  " ] 信息...");
         my $group_info = $self->_get_group_info($group->gcode);
         if(defined $group_info){
-            $self->debug("更新群 [ " . $group->gname .  " ] 信息成功(未获取到群成员信息)") 
+            $self->warn("更新群 [ " . $group->gname .  " ] 信息成功(未获取到群成员信息)") 
                 if ref $group_info->{member} ne "ARRAY";
             $group->update($group_info);
         }
@@ -183,7 +308,7 @@ sub update_group {
         return $self;
     }
     my @groups;
-    $self->debug("更新群列表信息...\n");
+    $self->info("更新群列表信息...\n");
     my $group_list = $self->_get_group_list_info(); 
     unless(defined $group_list){
         $self->warn("更新群列表信息失败\n");
@@ -200,7 +325,7 @@ sub update_group {
             $group_info = $g;
         }
         if(ref $group_info->{member} ne 'ARRAY'){
-            $self->debug("更新群 [ " . $group_info->{gname} .  " ]信息成功(未获取到群成员信息)");
+            $self->warn("更新群 [ " . $group_info->{gname} .  " ]信息成功(未获取到群成员信息)");
         }
         push @groups, $self->new_group($group_info);
     } 
@@ -245,6 +370,18 @@ sub search_group {
     }
 }
 
+sub remove_group_member{
+    my $self = shift;
+    my $member = shift;
+    $self->die("不支持的数据类型") if not $member->is_group_member;
+    for(my $i=0;$i<@{$member->group->member};$i++){
+        if($member->group->member->[$i]->id eq $member->id){
+            splice @{$member->group->member},$i,1;
+            return 1;
+        }
+    }
+    return;
+}
 sub search_group_member {
     my $self = shift;
     my %p = @_;
@@ -399,6 +536,124 @@ sub search_recent {
     }
 }
 
+sub invite_friend{
+    my $self = shift;
+    return if not $self->is_support_model_ext;
+    my $group = shift;
+    my @friends = @_;
+    if(not defined $group->gnumber){
+        $self->error("未获取到群号码，无法邀请好友入群");
+        return;
+    }
+    if($group->me->gtype ne "manage" and $group->me->gtype ne "create"){
+        $self->error("非群主或管理员，无法邀请好友入群");
+        return;
+    }
+    for(@friends){
+        $self->die("非好友对象") if not $_->is_friend;
+    }
+    my $ret = $self->_invite_friend($group->gnumber,map {$_->qq}  @friends);
+    if($ret){$self->info("邀请好友入群成功")}
+    else{$self->error("邀请好友入群失败")}
+    return $ret;
+}
+sub kick_group_member{
+    my $self = shift;
+    return if not $self->is_support_model_ext;
+    my $group = shift;
+    my @members = @_;
+    if(not defined $group->gnumber){
+        $self->error("未获取到群号码，无法踢除群成员");
+        return;
+    }
+    if($group->me->gtype ne "manage" and $group->me->gtype ne "create"){
+        $self->error("非群主或管理员，无法踢除群成员");
+        return;
+    }
+    for(@members){                                            
+        $self->die("非群成员对象") if not $_->is_group_member;  
+    }
+    my $ret = $self->_kick_group_member($group->gnumber,map {$_->qq} @members);
+    if($ret){
+        for(@members){
+            $self->remove_group_member($_);
+            $self->emit(lose_group_member=>$_);
+        }
+        $self->info("踢除群成员成功");
+    }
+    else{$self->error("剔除群成员失败")}
+    return $ret;
+}
+sub set_group_admin{
+    my $self = shift;
+    return if not $self->is_support_model_ext;
+    my $group = shift;
+    my @members = @_;
+    if(not defined $group->gnumber){
+        $self->error("未获取到群号码，无法设置管理员");
+        return;
+    }
+    if($group->me->gtype ne "create"){
+        $self->error("非群主，无法设置管理员");
+        return;
+    }
+    for(@members){                                            
+        $self->die("非群成员对象") if not $_->is_group_member;
+    }
+    my $ret = $self->_set_group_admin($group->gnumber,map {$_->qq} @members);
+    if($ret){
+        $_->role("admin") for(@members);
+        $self->info("设置管理员成功");
+    }
+    else{$self->error("设置管理员失败")}
+    return $ret;
+}
+sub remove_group_admin{
+    my $self = shift;
+    my $group = shift;
+    my @members = @_;
+    if(not defined $group->gnumber){
+        $self->error("未获取到群号码，无法移除管理员");
+        return;
+    }
+    if($group->me->gtype ne "create"){
+        $self->error("非群主，无法移除管理员");
+        return;
+    }
+    for(@members){
+        $self->die("非群成员对象") if not $_->is_group_member;
+    }
+    my $ret = $self->_remove_group_admin($group->gnumber,map {$_->qq} @members);
+    if($ret){
+        $_->role("member") for(@members);
+        $self->info("移除管理员成功");
+    }
+    else{$self->error("移除管理员失败")}
+    return $ret;
+}
+sub set_group_member_card{
+    my $self = shift;
+    my $group = shift;
+    my $member = shift;
+    my $card = shift;
+    if(not defined $group->gnumber){
+        $self->error("未获取到群号码，无法设置群名片");
+        return;
+    }
+    if(!$member->is_me and $group->me->gtype ne "manage" and $group->me->gtype ne "create"){
+        $self->error("非群主或管理员，无法设置其他人群名片");
+        return;
+    }
+    $self->die("非群成员对象") if not $member->is_group_member;
+    my $ret = $self->_set_group_member_card($group->gnumber,$member->qq,$card);
+    if($ret){
+        $member->card($card);
+        if(defined $card){$self->info("设置群名片成功");}
+        else{$self->info("取消群名片成功");}
+    }
+    else{$self->error("设置群名片失败")}
+    return $ret;
+}
 
 sub new_user{
     my $self = shift;
